Package: kdensity
Type: Package
Title: Kernel Density Estimation with Parametric Starts and Asymmetric
        Kernels
Version: 1.2.0
Authors@R: c(
    person("Jonas", "Moss", , "jonas.gjertsen@gmail.com", role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-6876-6964")),
    person("Martin", "Tveten", ,"tveten@nr.no", role = c("ctb"))
    )
Description: Handles univariate non-parametric density estimation with 
    parametric starts and asymmetric kernels in a simple and flexible way. 
    Kernel density estimation with parametric starts involves fitting a
    parametric density to the data before making a correction with kernel 
    density estimation, see Hjort & Glad (1995) <doi:10.1214/aos/1176324627>.
    Asymmetric kernels make kernel density estimation more efficient on bounded
    intervals such as (0, 1) and the positive half-line. Supported asymmetric 
    kernels are the gamma kernel of Chen (2000) <doi:10.1023/A:1004165218295>,
    the beta kernel of Chen (1999) <doi:10.1016/S0167-9473(99)00010-9>, and the
    copula kernel of Jones & Henderson (2007) <doi:10.1093/biomet/asm068>.
    User-supplied kernels, parametric starts, and bandwidths are supported.
License: MIT + file LICENSE
URL: https://github.com/JonasMoss/kdensity
BugReports: https://github.com/JonasMoss/kdensity/issues
Encoding: UTF-8
Suggests: extraDistr, testthat, knitr, rmarkdown
Imports: univariateML
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 15:20:52 UTC; jmoss
Author: Jonas Moss [aut, cre] (ORCID: <https://orcid.org/0000-0002-6876-6964>),
  Martin Tveten [ctb]
Maintainer: Jonas Moss <jonas.gjertsen@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 21:10:57 UTC
Built: R 4.6.0; ; 2026-05-05 00:20:57 UTC; unix
