# Generated by roxygen2: do not edit by hand

S3method("$",kdensity)
S3method("$<-",kdensity)
S3method("[[",kdensity)
S3method("[[<-",kdensity)
S3method(coef,kdensity)
S3method(lines,kdensity)
S3method(logLik,kdensity)
S3method(plot,kdensity)
S3method(points,kdensity)
S3method(print,kdensity)
S3method(summary,kdensity)
S3method(update,kdensity)
export(kdensity)
