% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kernels.R
\name{get_kernel}
\alias{get_kernel}
\title{Helper function that gets a kernel function for kdensity.}
\usage{
get_kernel(kernel_str)
}
\arguments{
\item{kernel_str}{a string specifying which kernel to use.}
}
\value{
a kernel function of the format k(u) with integral normalized
to 1.
}
\description{
Helper function that gets a kernel function for kdensity.
}
\keyword{internal}
